from django.core.management.base import BaseCommand
from bot.core import TelegramBot

class Command(BaseCommand):
    help = 'Runs the Telegram setting with optional auto-reload'

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):

        bot = TelegramBot()
        bot.run()




from asgiref.sync import sync_to_async


class PaginationResult:
    def __init__(self, items, total_pages, current_page, has_next, has_prev, total_items):
        self.items = items
        self.total_pages = total_pages
        self.current_page = current_page
        self.has_next = has_next
        self.has_prev = has_prev
        self.total_items = total_items


class PaginatorOutput:
    def __init__(self, instance, joiner='\n', close=True, btn_sizes=2):
        self.instance = instance
        self.joiner = joiner
        self.close = close
        self.btn_sizes = btn_sizes


class Paginator:
    def __init__(self, data, per_page=10, formatter=None, reverse=False):
        self._data = data
        self._per_page = per_page
        self.formatter = formatter
        self.reverse = reverse
        self.total_items = None
        self.last_fetched_page = None
        self.is_memory = isinstance(data, list)
        print(f"is_memory: {self.is_memory}")
        if self.is_memory:
            self.total_items = len(data)


    async def get_total_items(self):
        if self.is_memory:
            return self.total_items
        if self.total_items is None:
            self.total_items = await sync_to_async(self._data.count)()
        return self.total_items

    async def get_items(self, offset, limit):
        sort = -1 if self.reverse else None
        if self.is_memory:
            out=self._data[offset:offset + limit]
            if self.reverse:
                return list(reversed(out))
        return await sync_to_async(lambda: list(self._data[offset:offset + limit:sort]))()

    def set_last_fetched_page(self, **kwargs):
        self.last_fetched_page = PaginationResult(
            items=kwargs.get('items', []),
            total_pages=kwargs.get('total_pages', 0),
            current_page=kwargs.get('current_page', 0),
            has_next=kwargs.get('has_next', False),
            has_prev=kwargs.get('has_prev', False),
            total_items=kwargs.get('total_items', 0)
        )

    async def paginate(self, page: int = 1) -> PaginationResult:
        total_items = await self.get_total_items()
        total_pages = (total_items + self._per_page - 1) // self._per_page
        page = max(1, min(page, total_pages))

        offset = (page - 1) * self._per_page
        items = await self.get_items(offset, self._per_page)

        if self.formatter:
            items = [self.formatter(item) for item in items]

        result = PaginationResult(
            items=items,
            total_pages=total_pages,
            current_page=page,
            has_next=page < total_pages,
            has_prev=page > 1,
            total_items=total_items
        )

        self.last_fetched_page = result
        return result



from ..ext.base import Base
from bot.utils.setup.select_language import select_language_setup
class NodeUser(Base):
    def __init__(self):
        super().__init__('user',1)

    async def after_setup(self):
        if await select_language_setup(self):
            self.allow_continue=False
            return
        else:self.allow_continue=True


from ..ext.base import Base, MH
from ..models import Message


class NodeUser(Base):
    def __init__(self):
        super().__init__('user', 1)

    def n_active_service(self):
        e = MH()
        e.filters = self.match_filter(active_service)

        async def show_services():
            query = Order.objects.filter(user=self.user, status='complete', is_expired=False, parent_order=None)
            if await self.db(query.count) > 0:
                def formater(item):
                    return item.uuid[:5], {'act': 'show_order', 'order_id': item.id}

                self.pagination('active_service', query.all(), formater, per_page=10, btn_sizes=2)
                btn = await self.make_pagination_button('active_service', 1)
                # caption=active_service_text
                await self.send('services', reply_markup=btn)
            else:
                caption = no_active_service_text
                await self.send(caption)

        e.callback = show_services
        self.node(e)



from dataclasses import dataclass
from typing import Union, List, Dict, Any
from telegram import InlineKeyboardButton, InlineKeyboardMarkup, KeyboardButton, ReplyKeyboardMarkup,WebAppInfo


def chunks(input_data, sizes: int | list | tuple, reverse=False, append_last=True):
    """
    Splits a list or dictionary into sublists/subdicts based on the specified sizes.

    :param input_data: The list or dictionary to be split
    :param sizes: A tuple or list of integers representing the sizes of the sublists/subdicts
    :param reverse: If True, reverses each chunk
    :param append_last: If True, appends remaining elements as last chunk
    :return: A list of sublists/subdicts split according to the sizes
    """
    do_reverse = lambda x: x[::-1] if reverse else x

    # Handle dictionary input
    if isinstance(input_data, dict):
        items = list(input_data.items())
        chunks_result = chunks(items, sizes, reverse, append_last)
        return [dict(chunk) for chunk in chunks_result]

    # Original list handling
    if isinstance(sizes, int):
        res = []
        for i in range(0, len(input_data), sizes):
            res += [do_reverse(input_data[i:i + sizes])]
        return res

    result = []
    start_index = 0

    for size in sizes:
        end_index = start_index + size
        result.append(do_reverse(input_data[start_index:end_index]))
        start_index = end_index
    if append_last and start_index < len(input_data):
        result.append(do_reverse(input_data[start_index:]))

    return result


import json
@dataclass
class ButtonBuilder:
    data: Union[List[str], Dict]
    sizes: Union[int, List, tuple]
    reverse: bool = False
    close: bool = True


    def build(self, **kwargs) -> Union[ReplyKeyboardMarkup, InlineKeyboardMarkup]:
        """
            Builds and returns a Telegram keyboard based on input data type.

            Args:
                **kwargs: Configuration options for keyboard creation
                    - resize: Allow keyboard resizing (default: True)
                    - share_phone: Add phone sharing button (default: False)
                    - KeyboardButtonKwargs: Additional args for KeyboardButton
                    - ReplyKeyboardMarkupKwargs: Additional args for ReplyKeyboardMarkup
                    - close_callback_data: Custom callback data for close button (default: 'close')
                    - close_title: Custom title for close button (default: 'close')

            Returns:
                ReplyKeyboardMarkup: If input data is a list
                InlineKeyboardMarkup: If input data is a dictionary

            Examples:
                # Create reply keyboard
                kb = KeyboardBuilder(['btn1', 'btn2'], sizes=2).build(resize=True)

                # Create inline keyboard
                kb = KeyboardBuilder({'Button': 'callback_data'}, sizes=1).build(
                    close_title='Close Menu',
                    close_callback_data='menu_close'
                )
            """
        if isinstance(self.data, list):
            return self._build_reply_keyboard(**kwargs)
        return self._build_inline_keyboard(**kwargs)

    def _build_reply_keyboard(self, **kwargs) -> ReplyKeyboardMarkup:
        resize = kwargs.get('resize', True)
        share_phone = kwargs.get('share_phone', False)
        button_kwargs = kwargs.get('KeyboardButtonKwargs', {})
        markup_kwargs = kwargs.get('ReplyKeyboardMarkupKwargs', {})

        data = self._chunk_data(self.data.copy())
        buttons = [
            [KeyboardButton(col, request_contact=share_phone, **button_kwargs)
             for col in row]
            for row in data
        ]
        return ReplyKeyboardMarkup(buttons, resize, **markup_kwargs)

    def _build_inline_keyboard(self, **kwargs) -> InlineKeyboardMarkup:
        close_data = kwargs.get('close_callback_data',json.dumps({'act':'close'}) )
        close_title = kwargs.get('close_title', 'close')
        buttons = []
        for key, value in self.data.items():
            if isinstance(value, str):
                if any(prefix in value for prefix in ('http://', 'https://')):
                    buttons.append(InlineKeyboardButton(key, url=value))
                else:
                    raise ValueError(f"Invalid value type in data\n{value}")
                    # buttons.append(InlineKeyboardButton(key, callback_data=value))
            elif isinstance(value, dict):
                json_data = json.dumps(value)
                if len(json_data)>64:
                    raise ValueError(f"data len more 64 bytes\n{value}")
                buttons.append(InlineKeyboardButton(key,callback_data=json_data))
            elif isinstance(value, WebAppInfo):
                buttons+=[InlineKeyboardButton(key,web_app=value)]
            else:
                raise ValueError(f"Invalid value type in data\n{value}")
        if self.close:
            buttons.append(InlineKeyboardButton(close_title, callback_data=close_data))
        return InlineKeyboardMarkup(self._chunk_data(buttons))

    def _chunk_data(self, data: List) -> List:
        if isinstance(self.sizes, int) and self.sizes == 0:
            return [data]

        return chunks(data, self.sizes, reverse=self.reverse)




from ..ext.base import Base,MH,QH
from ..models import Message
class NodeUser(Base):
    def __init__(self):
        super().__init__('user',1)

    def n_test(self):
        e = MH()
        e.filters = self.match_filter('n_test')
        async def act():
            def formater(item):
                return {'key': item.pk, 'data': item.message_id}
            self.pagination('test', Message.objects.all(), formater, 11, '\n-------------\n', True)
            msg,btn = await self.make_pagination_pm('test', 1)
            await self.bot.send_message(self.update.effective_chat.id, msg, reply_markup=btn)

        e.callback = act


import os

from dotenv import load_dotenv
load_dotenv()
from telegram import Bot, Update, BotCommand, BotCommandScopeChat
from functools import wraps
from .handler import QH, MH, Conversation
from asgiref.sync import sync_to_async
from django.core.cache import cache
from typing import (
    Any,
    Optional,
)

from .filters import LevelFilter, MatchText,IsConversation
from telegram.ext import filters, ConversationHandler
from bot.component.button import ButtonBuilder
from bot.component.paginator import Paginator, PaginationResult, PaginatorOutput
from django.core.management import call_command
from bot.models import User
from django.contrib.staticfiles import finders
from pathlib import Path
import json
from bot.models import Setting
from ._base import BaseBot
from .middlewares import check_status_middleware, check_maintenance_middleware


class Base(BaseBot):
    DEFAULT_FALLBACK = "default_fallback"
    END_CONVERSATION = 'end'
    def __init__(self, name: str):
        super().__init__(name)
        self.__commands = []
        self.__middlewares:list=[]




    def build(self, bot,  admins):
        self.bot = bot
        self.admins = admins
        self.default_filters = self.chat_filters & LevelFilter(self, self.admins)
        self.__collect_nodes()
        self.__default_nodes()
        self.__not_found()
        self.add_middleware(check_status_middleware)
        self.add_middleware(check_maintenance_middleware)
        del self.entry_MH_filters


    def __not_found(self):
        e = MH()
        e.filters = filters.ALL

        async def act():
            # await self.menu_action()
            if self.update.effective_user.is_bot:
                return 'end'
            await self.send(self.T('not_found_text'), reply_markup=self.menu_button())
            return 'end'

        e.callback = act
        self.node(e)


    def __collect_nodes(self):
        methods_names = self.__get_methods()
        for i in methods_names:
            self.__getattribute__(i)()

    def __user_data(self, key, value=None):
        if value:
            self.context.user_data[key] = value
        else:
            return self.context.user_data.get(key)

    def __decorator(self, func, *args, **kwargs):
        @wraps(func)
        async def wrapper(*args, **kwargs):
            self.update = args[0]
            self.context = args[1]
            await self.__set_user(self.update.effective_user.id)
            await self.__set_setting()

            await self.__setup()

            async def off():
                return -1

            if self.allow_continue:
                out = await func()
                func_name = func.__name__
                sep_name = Base.__decode_idx(func_name)
                try:
                    print(f"set is_conversation for {func_name}")
                    self.context.user_data['is_conversation'] = True if 'isconversation' in sep_name[0] else False
                except Exception as e:
                    pass
                if out == None:
                    if str(sep_name[-2]) != "end":
                        out = f"{self._name}_{sep_name[-2]}"
                        self.now_state(out)
                    else:
                        out = ConversationHandler.END
                if out == "end":
                    out = ConversationHandler.END
                return out
            else:
                out = await off()
                return out

        return wrapper

    def __init_attr(self):
        self.user_id = self.update.effective_user.id
        self.chat_id = self.update.effective_chat.id
        self.message_id = self.update.effective_message.message_id
        self.text = self.update.effective_message.text or self.update.effective_message.caption
        if self.update.effective_message.caption:
            self.message_type = 'media'


    async def __setup(self):
        print('setup')
        self.__init_attr()
        self.language = self.user.language or os.getenv('DEFAULT_LANGUAGE')
        self.T.language=self.language
        for i in self.__middlewares:
            if not await i(self):
                return

    @sync_to_async
    def __set_user(self, user_id: int):
        self._set_users_level()
        user_cache_key = f'{user_id}:user'
        user = cache.get(user_cache_key)
        if user:
            self.user = user
            return
        if not user:
            user = User.objects.filter(user_id=user_id).first()
        if user:
            if self.update:
                update_user_data = self.update.effective_user
                first_name = update_user_data.first_name
                username = update_user_data.username
                last_name = update_user_data.last_name
                user.username = username
                user.last_name = last_name
                user.first_name = first_name
                user.is_active = True
                user.save()
        else:
            User(
                user_id=user_id,
                is_active=True,
            ).save()
            user = User.objects.filter(user_id=user_id).first()
        if user:
            cache.set(user_cache_key, user, 15)
        self.user = user

    @sync_to_async
    def __set_setting(self):
        self.setting = Setting.get_setting()


    @sync_to_async
    def __set_referrer(self, referrer_id: int):
        """
        Set the referrer for the current user
        Args:
            referrer_id: Telegram user_id of the referrer
        """
        if not self.user.referred_by:
            referrer = User.objects.filter(user_id=referrer_id).first()
            if referrer and referrer.id != self.user.id:
                self.user.referred_by = referrer
                self.user.save()
                return True
        return False

    def __get_methods(self):
        return [method for method in dir(self) if
                method.startswith(self.method_prefix) and callable(getattr(self, method))]

    @staticmethod
    def __idx_maker(name, idx_node, idx_items, idx_item,is_conversation):
        add_is_conversation='isconversation' if is_conversation else ''
        out=  f'{name}{add_is_conversation}_{idx_node}_{idx_items}_{idx_item}'
        # print(out)
        return out

    @staticmethod
    def __decode_idx(name):
        return name.split('_')

    def __process_handler(self, handler: list | QH | MH | Any, states_level=False, start_from=0,is_conversation=False):
        if type(handler) == type(None): return
        if not isinstance(handler, list):
            handler = [handler]
        out = []
        idx_node = len(self._handlers)
        for base_items_level, items in enumerate(handler):
            items_level = base_items_level + start_from
            temp = []
            if not isinstance(items, list):
                items = [items]
            for i, item in enumerate(items):
                callback = item.callback
                if type(callback) != type(None):
                    # process callback
                    if states_level and base_items_level == len(handler) - 1:
                        items_level = 'end'
                    callback_name = Base.__idx_maker(self._name, idx_node, items_level, i,is_conversation=is_conversation)
                    callback.__name__ = callback_name
                    item.callback = self.__decorator(callback)

                if type(item.filters) != type(None):
                    # process filters
                    if type(item) == QH:
                        filter_text = item.filters
                        if not isinstance(filter_text, str):
                            raise Exception(f'bad filter: {filter_text}')
                        item.filters = self.inline_filter(filter_text, item._exact_match, self._level,is_conversation=is_conversation)
                    else:
                        if isinstance(item.filters, str):
                            item.filters = self.match_filter(item.filters)
                        # if not states_level:
                        #     self.entry_MH_filters += [item.filters]
                        item.filters = item.filters & self.default_filters
                        if states_level:
                            item.filters = item.filters & ~self.match_filter(self.T('menu_btn')) & IsConversation(self,is_conversation)
                if states_level:
                    temp.append(item.build())
                else:
                    out.append(item.build())
            if temp:
                out.append(temp)
        # print(out)
        return out

    def __default_fallback(self):
        e = MH()
        e.filters = self.match_filter(self.T('menu_btn'))

        async def act():
            self.kill_state()
            await self.send(self.T('menu_text'), reply_markup=self.menu_button())
            return 'end'

        e.callback = act
        return e

    def __default_nodes(self):
        back_menu_node = MH()
        back_menu_node.filters = self.match_filter(self.T('menu_btn'))

        async def act():
            await self.menu_action()
            return 'end'

        back_menu_node.callback = act
        self.node(back_menu_node)
        ######################################
        close_node = QH()
        close_node.filters = 'close'

        async def act():
            await self.bot.deleteMessage(self.update.effective_chat.id, self.update.effective_message.message_id)
            return 'end'

        close_node.callback = act
        self.node(close_node)
        ######################################
        next_page_node = QH()
        next_page_node.filters = 'next_page'

        async def act():
            paginator_name = self.queries('name')
            last_fetched_page = self.get_last_fetched_page(paginator_name)
            msg, btn = await self.make_pagination_pm(paginator_name, last_fetched_page.current_page + 1)
            await self.bot.editMessageText(msg, self.update.effective_chat.id, self.update.effective_message.message_id,
                                           reply_markup=btn)
            return 'end'

        next_page_node.callback = act
        self.node(next_page_node)
        ######################################
        prev_page_node = QH()
        prev_page_node.filters = 'prev_page'

        async def act():
            paginator_name = self.queries('name')
            last_fetched_page = self.get_last_fetched_page(paginator_name)
            msg, btn = await self.make_pagination_pm(paginator_name, last_fetched_page.current_page - 1)
            await self.bot.edit_message_text(msg, self.update.effective_user.id,
                                             self.update.effective_message.message_id, reply_markup=btn)
            return 'end'

        prev_page_node.callback = act
        self.node(prev_page_node)
        ######################################
        prev_page_btn_node = QH()
        prev_page_btn_node.filters = 'prev_page_btn'

        async def act():
            paginator_name = self.queries('name')
            last_fetched_page = self.get_last_fetched_page(paginator_name)
            btn = await self.make_pagination_button(paginator_name, last_fetched_page.current_page - 1)
            await self.bot.edit_message_reply_markup(self.update.effective_user.id,
                                                     self.update.effective_message.message_id, reply_markup=btn)
            return 'end'

        prev_page_btn_node.callback = act
        self.node(prev_page_btn_node)
        ######################################
        next_page_btn_node = QH()
        next_page_btn_node.filters = 'next_page_btn'

        async def act():
            paginator_name = self.queries('name')
            last_fetched_page = self.get_last_fetched_page(paginator_name)
            btn = await self.make_pagination_button(paginator_name, last_fetched_page.current_page + 1)
            await self.bot.edit_message_reply_markup(self.update.effective_user.id,
                                                     self.update.effective_message.message_id, reply_markup=btn)
            return 'end'

        next_page_btn_node.callback = act

        self.node(next_page_btn_node)
        ######################################
        makemessages_node = MH()
        makemessages_node.filters = self.match_filter('/compile')

        async def act():
            if self.__translation_manager:
                for lang in self.__translation_manager.languages:
                    await sync_to_async(call_command)(
                        'compilemessages',
                        verbosity=1
                    )
                await self.send('done')
            else:
                await self.send('Translation Manager not Set')
            return 'end'

        makemessages_node.callback = act
        self.node(makemessages_node)
        ######################################
        select_language_node = QH()
        select_language_node.filters = 'select_language'

        async def act():
            lang = self.queries('lang')
            self.user.language = lang
            self.user.empty_cache()
            await sync_to_async(self.user.save)()
            await self.alert(self.T('success_select_language_text'), t=True)
            await self.bot.deleteMessage(self.chat_id, self.message_id)
            await self.start_action()
            return 'end'

        select_language_node.callback = act
        self.node(select_language_node)

    def state_data(self, main_key, key, value=None, do_empty=False):
        if do_empty:
            self.context.user_data[main_key] = {}

        old_data = self.__user_data(main_key) or {}
        if value is not None:
            old_data[key] = value
            self.__user_data(main_key, old_data)
        else:
            return old_data.get(key)

    def now_state(self, value=None):
        key = '_now_state'
        return self.__user_data(key, value)

    def go_state(self, state):
        if now_state := self.now_state():
            try:
                now_state_int = int(now_state.replace(f'{self._name}_', ''))
                new_state_int = now_state_int + state
                if new_state_int < 0:
                    out = f"{self._name}_" + str(now_state_int)
                else:
                    out = f"{self._name}_" + str(new_state_int)
                self.now_state(out)
                return out
            except Exception as e:
                print(e)

    def kill_state(self):
        key = '_now_state'
        self.context.user_data[key]=None

    def inline_filter(self, filter_text, exact: bool, level: int,is_conversation,*args):
        def checker(*args):
            callback_data = json.loads(args[0])
            if exact:
                if filter_text != callback_data['act']:return False
            else:
                if filter_text not in args[0]: return False
            try:
                print(filter_text,callback_data['act'],self.context.user_data.get('is_conversation',True) ,is_conversation)
                if self.context.user_data.get('is_conversation',True) != is_conversation:
                    return False
            except Exception as e:
                pass
            if level != self._level:
                return False
            if self.user and self.user.level != self._level:
                return False

            R = callback_data.get('R')
            if R and R != self._level:
                return False
            return True


        return checker

    async def send(self,
                   text: str,
                   chat_id: int | str = None,
                   reply_markup=None,
                   parse_mode=None,
                   disable_notification=None,
                   t=True,
                   **kwargs):
        if t:
            text = self.T(text)
        if not chat_id:
            chat_id = self.chat_id
        kwargs['chat_id'] = chat_id
        kwargs['text'] = text
        kwargs['reply_markup'] = reply_markup
        kwargs['parse_mode'] = parse_mode
        kwargs['disable_notification'] = disable_notification

        return await self.bot.send_message(**kwargs)

    async def edit(self,
                   text: str,
                   message_id: int = None,
                   chat_id: int | str = None,
                   reply_markup=None,
                   parse_mode=None,
                   t=True,
                   **kwargs):
        text = self.T(text) if t else text
        kwargs['chat_id'] = chat_id or self.chat_id
        kwargs['message_id'] = message_id or self.message_id
        kwargs['text'] = text
        kwargs['reply_markup'] = reply_markup
        kwargs['parse_mode'] = parse_mode
        return await self.bot.edit_message_text(**kwargs)

    async def edit_caption(self,
                           new_caption: str,
                           message_id: int = None,
                           chat_id: int | str = None,
                           reply_markup=None,
                           parse_mode=None,
                           t=True,
                           **kwargs):
        if t:
            new_caption = self.T(new_caption)
        if not chat_id:
            chat_id = self.chat_id
        if not message_id:
            message_id = self.message_id
        kwargs['chat_id'] = chat_id
        kwargs['message_id'] = message_id
        kwargs['caption'] = new_caption
        kwargs['reply_markup'] = reply_markup
        kwargs['parse_mode'] = parse_mode
        return await self.bot.edit_message_caption(**kwargs)

    async def send_media(self,
                         media: str | bytes,
                         media_type: str = 'photo',
                         caption=None,
                         chat_id: int | str = None,
                         reply_markup=None,
                         parse_mode=None,
                         disable_notification=None,
                         t=True,
                         **kwargs):
        if t:
            caption = self.T(caption)
        if not chat_id:
            chat_id = self.chat_id
        kwargs['chat_id'] = chat_id
        kwargs[media_type] = media
        kwargs['caption'] = caption
        kwargs['reply_markup'] = reply_markup
        kwargs['parse_mode'] = parse_mode
        kwargs['disable_notification'] = disable_notification
        return await getattr(self.bot, f'send_{media_type}')(**kwargs)

    async def copy(self,
                   from_chat_id: int | str,
                   message_id: int,
                   chat_id: int | str = None,
                   reply_markup=None,
                   disable_notification=None,
                   **kwargs):
        kwargs['chat_id'] = chat_id or self.chat_id
        kwargs['from_chat_id'] = from_chat_id
        kwargs['reply_markup'] = reply_markup
        kwargs['message_id'] = message_id
        kwargs['disable_notification'] = disable_notification
        return await self.bot.copy_message(**kwargs)

    async def delete(self,
                     chat_id: str | int = None,
                     message_id: int = None,
                     **kwargs):

        kwargs['chat_id'] = chat_id or self.chat_id
        kwargs['message_id'] = message_id or self.message_id
        return await self.bot.deleteMessage(**kwargs)

    def match_filter(self, text, exact=True):
        return MatchText(text, self, exact)

    def node(self,
             entry_points: list | QH | MH | Any,
             states: Optional[list[QH | MH | Any]] = None,
             fallbacks: Optional[list | QH | MH | Any] = DEFAULT_FALLBACK,
             kwargs: dict[str, Any] = {},
             ):
        is_conversation=type(states) != type(None)

        entry_points = self.__process_handler(entry_points,is_conversation=is_conversation)
        states = self.__process_handler(states, states_level=True, start_from=1,is_conversation=is_conversation)
        if fallbacks == 'default_fallback':
            fallbacks = [self.__default_fallback()]
        fallbacks = self.__process_handler(fallbacks, start_from=2,is_conversation=is_conversation)
        handlers_obj = Conversation.make(self._name, entry_points, states, fallbacks, kwargs)
        for handler_obj in handlers_obj:
            self._handlers.append(handler_obj)

    def button(self, data: list | dict, pattern: int | tuple, reverse: bool = False, close: bool = False, t=True):
        new_data = data.copy()
        if isinstance(data, dict):
            for key, value in data.items():
                if isinstance(value, dict):
                    if 'R' not in value:
                        value['R'] = self._level
                    new_data[key] = value
        if t:
            if isinstance(data, dict):
                new_data = {self.T(k): v for k, v in new_data.items()}
            elif isinstance(data, list):
                new_data = [self.T(i) for i in new_data]
        return ButtonBuilder(new_data, pattern, reverse, close).build()

    def is_query(self) -> bool:
        if self.update.callback_query:
            return True
        return False

    def queries(self, key=None):
        q = json.loads(self.update.callback_query.data)
        if key:
            return q.get(key)
        return q

    def pagination(self, name: str, queryset, formatter_function=None, per_page: int = 10, joiner: str = '\n',
                   close: bool = True, btn_sizes=2, reverse=True):
        """
            Make pagination for telegram messages
            For Buttons Formatter must be return Dict like {key: 'key',data:'value'}
        """
        instance = Paginator(queryset, per_page=per_page, formatter=formatter_function, reverse=reverse)
        self.__pagination[name] = PaginatorOutput(
            instance=instance,
            joiner=joiner,
            close=close,
            btn_sizes=btn_sizes,
        )

    def get_paginator(self, name):
        paginator = self.__pagination.get(name)
        if not paginator:
            raise Exception('Pagination not initialized')
        return paginator

    async def get_page(self, name: str, page_number: int):
        page: PaginationResult = await self.get_paginator(name).instance.paginate(page_number)
        return page

    def get_last_fetched_page(self, name: str) -> PaginationResult:
        return self.get_paginator(name).instance.last_fetched_page

    async def make_pagination_pm(self, name: str, page_number=1):
        page = await self.get_page(name, page_number)
        paginator = self.get_paginator(name)
        msg = paginator.joiner.join(page.items)
        btn = {}
        if page.has_prev:
            btn[self.T('prev_btn')] = {'act': 'prev_page', 'name': name}
        if page.has_next:
            btn[self.T('next_btn')] = {'act': 'next_page', 'name': name}
        x = 2 if len(btn) == 2 else 1
        btn = self.button(btn, (x, 1), close=paginator.close)
        return msg, btn

    async def make_pagination_button(self, name: str, page_number=1):
        page = await self.get_page(name, page_number)
        paginator = self.get_paginator(name)
        btn_sizes = paginator.btn_sizes
        btn = {}
        for i in page.items:
            btn[str(i[0])] = i[1]
        action_btn = {}
        if page.has_prev:
            action_btn[self.T('prev_bn')] = {'act': 'prev_page_btn', 'name': name}
        if page.has_next:
            action_btn[self.T('next_btn')] = {'act': 'next_page_btn', 'name': name}
        p1 = len(btn) // btn_sizes
        p2 = len(btn) % btn_sizes
        sizes = [btn_sizes] * (p1)
        if p2 > 0:
            sizes.append(p2)
        x = 2 if len(action_btn) == 2 else 1
        sizes += [x, 1]
        btn = {**btn, **action_btn}
        btn = self.button(btn, tuple(sizes), close=paginator.close)
        return btn

    async def alert(self, msg, id=0, popup=False, t=False):
        if t:
            msg = self.T(msg)
        if id == 0:
            id = self.update.callback_query.id
        await self.bot.answer_callback_query(
            callback_query_id=id, text=msg, show_alert=popup)

    async def menu_action(self):
        await self.send(self.T('menu_text'), reply_markup=self.menu_button())

    async def start_action(self):
        await self.set_commands()
        btn = self.button(self.menu_btns, self.menu_btns_pattern)
        await self.send(self.T('welcome_text'), reply_markup=btn)

    def get_static_file_bytes(self, static_path):
        # Find the absolute path of the static file
        file_path = finders.find(static_path)
        if file_path:
            # Read file as bytes
            return Path(file_path).read_bytes()
        return None

    def menu_button(self):
        return self.button(self.menu_btns, self.menu_btns_pattern)

    def back_menu_button(self):
        return self.button([self.T('menu_btn')], 1)

    async def toggle(self, queryset, field: str):
        new_value = not getattr(queryset, field)
        setattr(queryset, field, new_value)
        await sync_to_async(queryset.save)()

    async def download(self, effective_message_file, output_path):
        file = effective_message_file
        file_id = file.file_id
        res = await self.bot.get_file(file_id)
        await res.download_to_drive(output_path)

    def add_command(self, command, description):
        self.__commands.append(BotCommand(command, description))

    async def set_commands(self):
        print('Setting commands')
        await self.bot.set_my_commands(self.__commands, BotCommandScopeChat(self.chat_id))

    def add_middleware(self, middleware_func):
        self.__middlewares.append(middleware_func)






from django.contrib import admin
from . import models


# Register your models here.
@admin.register(models.Setting)
class SettingAdmin(admin.ModelAdmin):
    fields = [field.name for field in models.Setting._meta.fields if field.name not in ['list_display_links', 'id']]
    list_display = fields
    list_editable = [i for i in fields if i != 'name']

    def has_add_permission(self, request, obj=None):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    class Media:
        css = {
            'all': ('setting/admin/css/style.css',),  # آدرس فایل CSS
        }
        js = ('setting/admin/js/script.js',)  # آدرس فایل JS


@admin.register(models.User)
class UserAdmin(admin.ModelAdmin):
    list_display = ('user_id', 'username', 'first_name', 'last_name', 'language', 'created_at', 'updated_at')
    list_filter = ('language', 'created_at')
    search_fields = ('user_id', 'username', 'first_name', 'last_name')
    ordering = ('-created_at',)




from django.shortcuts import render, redirect
import json
import telegram
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .core import TelegramBot
from colorama import init, Fore
from django.apps import apps
init(autoreset=True)
async def get_bot():
    if apps.ready:
        print(Fore.GREEN + 'get_bot')
        bot = TelegramBot()
        bot.app.updater=None
        bot.autodiscover()
        await bot.app.initialize()
        return bot
    return None

telegram_bot = None
@csrf_exempt
async def webhook(request, **kwargs):
    global telegram_bot
    if request.method == 'POST':
        try:
            if not telegram_bot:
                telegram_bot=await get_bot()
            update = telegram.Update.de_json(json.loads(request.body), telegram_bot.bot)
            await telegram_bot.app.process_update(update)
            return JsonResponse({"ok": True})
        except Exception as e:
            return JsonResponse({"ok": False, "error": str(e)})
    return JsonResponse({"ok": False})




import os
from django.core.management.base import BaseCommand
import telegram
from asgiref.sync import async_to_sync
from django.urls import reverse

class Command(BaseCommand):
    help = 'Sets webhook for Telegram setting'

    async def handle_async(self, *args, **options):
        WEBHOOK_HOST = os.getenv('WEBHOOK_URL')
        TELEGRAM_BOT_TOKEN = os.getenv('BOT_TOKEN')
        WEBHOOK_PORT = int(os.getenv('WEBHOOK_PORT'))
        bot = telegram.Bot(token=TELEGRAM_BOT_TOKEN)
        webhook_url = f"{WEBHOOK_HOST.rstrip('/')}{reverse('bot:webhook',kwargs={'token':TELEGRAM_BOT_TOKEN})}"
        self.stdout.write(self.style.WARNING(f'webhook url: {webhook_url}'))
        await bot.delete_webhook()
        await bot.set_webhook(webhook_url)

        self.stdout.write(self.style.SUCCESS(f'Webhook set to: {webhook_url}'))

        os.system(f'uvicorn --host 0.0.0.0 --port {WEBHOOK_PORT} --workers 1 --log-level info core.asgi:application')

    def handle(self, *args, **options):
        return async_to_sync(self.handle_async)(*args, **options)



from ..core.base import Base
from bot.utils.mixin.is_chat_member import ChatMemberCheck,IsChatMemberMixin,ChatInfo

class NodeUser(Base,IsChatMemberMixin):
    def __init__(self):
        super().__init__('user',1)
        self.chats_must_be_joined = [
            ChatInfo('🌟 کانال اطلاع رسانی 🌟', '@yozdlb'),
        ]

    async def after_setup(self):
        print('after_setupafter_setupafter_setup')
        c = ChatMemberCheck(self.chats_must_be_joined, self)
        result = await c.check(
            '✨ جهت اطلاع از اخرین اخبار ربات عضو شوید.',
            '✅ عضو شدم',
            'start',
            self.update.effective_user.id
        )
        if not result:
            self.allow_continue = False
            return
        else:
            self.allow_continue = True


from telegram.ext import (
    CallbackQueryHandler,
    ConversationHandler,
    MessageHandler,
    filters,
)


class HandlerMaker:
    def __init__(self):
        self.handler = None
        self.kwargs = {}
        self.filters = None
        self.callback = None

    def build(self):
        attrs = self.__dict__.copy()
        self.handler = attrs.get('_handler')
        forced_keys = attrs.get('_forced_keys', [])
        replace = attrs.get('_replace', {})
        extra_args = attrs.get('kwargs', {})

        # Clean up internal attributes
        for internal in ['_handler', 'handler', '_forced_keys', '_replace', 'kwargs']:
            attrs.pop(internal, None)

        # Validate required keys
        for key in forced_keys:
            if attrs.get(key) is None:
                raise ValueError(f'Required parameter {key} not set')

        # Process attributes
        for key, value in attrs.items():
            if not key.startswith('_'):
                target_key = replace.get(key, key)
                self.kwargs[target_key] = value

        self.kwargs.update(extra_args)
        # print(self.kwargs)
        return self.handler(**self.kwargs)


class QH(HandlerMaker):
    def __init__(self):
        super().__init__()
        self._exact_match = True
        self._handler = CallbackQueryHandler
        self._replace = {'filters': 'pattern'}
        self._forced_keys = ['callback', 'filters']


class MH(HandlerMaker):
    def __init__(self):
        super().__init__()
        self._handler = MessageHandler
        self._forced_keys = ['callback', 'filters']

from typing import (
    Any,
    Optional,
)


class Conversation:
    @staticmethod
    def make(name: str, entry_points: list | QH | MH | Any,
             states: Optional[list[QH | MH | Any]] = None,
             fallbacks: Optional[list | QH | MH | Any] = None,
             kwargs: dict[str, Any] = {}):
        if type(states) == type(None):
            return entry_points
        else:
            new_states = {}
            if not isinstance(states, list):
                states = [states]
            for i, state in enumerate(states):
                if not isinstance(states, list):
                    state = [state]
                new_states[f"{name}_{i}"] = state

            if type(fallbacks) == type(None):
                fallbacks = []
            if "allow_reentry" not in kwargs:
                kwargs['allow_reentry'] = True
            return [ConversationHandler(
                entry_points=entry_points,
                states = new_states,
                fallbacks=fallbacks,
                **kwargs
            )]



from django.db.models.signals import post_migrate
from django.dispatch import receiver
from .models import Setting

@receiver(post_migrate)
def create_default_record(sender, **kwargs):
    Setting.initialize()




from django.contrib.auth.models import AbstractBaseUser
from django.core.cache import cache
from django.conf import settings
import logging
from django.db import models
import os
from dotenv import load_dotenv
load_dotenv()

logger = logging.getLogger(__name__)

class Setting(models.Model):
    key='bot_setting'
    name = models.CharField(max_length=100, default=key)
    status_bot = models.BooleanField(default=True)
    is_maintenance = models.BooleanField(default=False)

    @classmethod
    def get_boolean_fields(cls):
        boolean_fields = {
            field.name: (field.value_from_object(cls.objects.first()), field.verbose_name or field.name)
            for field in cls._meta.fields
            if isinstance(field, models.BooleanField)
        }
        return boolean_fields

    @classmethod
    def get_setting(cls):
        if cache.get(cls.key):
            return cache.get(cls.key)
        out= cls.objects.first()
        cache.set(cls.key, out, 3600 * 24)
        return out

    @classmethod
    def initialize(cls):
        cls.objects.exclude(name=cls.key).delete()
        cache.delete(cls.key)
        setting, created = cls.objects.get_or_create(
            name=Setting.key,
            defaults={}
        )

        return setting

    def save(self, *args, **kwargs):
        cache.delete(self.name)
        super().save(*args, **kwargs)


class User(models.Model):
    user_id = models.BigIntegerField(unique=True, db_index=True)
    username = models.CharField(max_length=255, blank=True, null=True)
    first_name = models.CharField(max_length=255, blank=True, null=True)
    last_name = models.CharField(max_length=255, blank=True, null=True)
    is_active = models.BooleanField(default=True)
    is_banned = models.BooleanField(default=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    LANGUAGES = os.getenv('LANGUAGES').split(' ') if os.getenv('LANGUAGES') else ['en']
    BOT_LANGUAGES = [[i,i] for i in LANGUAGES]
    language = models.CharField(max_length=20, choices=BOT_LANGUAGES, null=True,blank=True)
    LEVELS = os.getenv('LEVELS').split(' ') if os.getenv('LEVELS') else ['admin']
    BOT_LEVELS = [[lvl,lvl] for lvl in LEVELS]
    level = models.CharField(max_length=20, choices=BOT_LEVELS, default=os.getenv('DEFAULT_LEVEL'))

    def __str__(self):
        name = self.username
        if name: return name[:10]
        name = self.first_name
        if name: return name[:10]
        name = str(self.user_id)
        if name: return name[:10]

    def empty_cache(self):
        cache.delete(f'{self.user_id}:user')

    def save(self, *args, **kwargs):
        try:
            old_level = cache.get('users_level').get(self.user_id)
            if old_level != self.level:
                print("level changed")
                cache.delete(f'users_level')
        except:
            pass
        super().save(*args, **kwargs)
        self.empty_cache()




from ._base import BaseBot
async def check_maintenance_middleware(self:BaseBot):
    if self.user_id in self.admins: return True
    self.allow_continue = not self.setting.is_maintenance
    if not self.setting.is_maintenance:
        return True
    else:
        await self.send(self.T('is_maintenance_text'))
        return False
async def check_status_middleware(self:BaseBot):
    if self.user_id in self.admins:
        return True
    self.allow_continue = self.setting.status_bot
    if self.setting.status_bot:
        return True
    else:
        await self.send(self.T('bot_is_offline_text'))
        return False



from telegram.ext._callbackcontext import CallbackContext
from telegram import Bot, Update
from django.core.cache import cache
from bot.component.paginator import PaginatorOutput
from bot.models import User
import os,json,logging
from dotenv import load_dotenv
from django.conf import settings
load_dotenv()
from telegram.ext import filters
from glob import glob
from pathlib import Path


class Texts:
    def __init__(self,languages,language):
        self.languages=languages
        self.language=language
        self.texts={}
        self.load_texts()

    def load_texts(self)->None:
        if cached:=cache.get(f'bot_texts', None):
            self.texts = cached
            return
        django_root = Path(settings.BASE_DIR)
        bot_panel_dir = django_root /os.getenv('PANEL_DIR')
        out = {}
        for lang in self.languages:
            data={}
            p = bot_panel_dir/f'texts/{lang}'
            for i in glob(f'{p}/*.json'):
                with open(i, 'r') as f:
                    items = f.read()
                    data.update(json.loads(items))
            out[lang] = data
        cache.set(f'bot_texts', out, timeout=60)
        self.texts = out

    def __call__(self, key):
        out= self.texts[self.language].get(key, key)
        return out


class BaseBot:
    def __init__(self, name):
        self._name = name
        self._level = name
        self.bot: Bot = None
        self.update: Update = None
        self.context: CallbackContext = None
        self.user: User = None
        self.setting = None
        self.method_prefix = os.getenv('BOT_METHODE_PREFIX') or "n_"
        self.menu_btns = []
        self.menu_btns_pattern = 3
        self.admins = []
        self.chat_filters = filters.ChatType.PRIVATE
        self.allow_continue = True
        self._handlers = []
        self._set_users_level()
        self.__pagination: dict[str, PaginatorOutput] = {}
        self.chat_id = None
        self.user_id = None
        self.message_id = None
        self.text = None
        self.message_type = 'text'
        self.languages = User.LANGUAGES
        self.language = os.getenv('DEFAULT_LANGUAGE')
        self.T: Texts = Texts(self.languages,self.language)

        self.entry_MH_filters = []
    def _set_users_level(self,empty_cache=False):
        if empty_cache:cache.delete('users_level')
        users_level = cache.get(f'users_level')
        if not users_level:
            try:
                print('run set_users_level')
                users_level = User.objects.all().values_list('user_id', 'level')
                if users_level:
                    users_level = {user_id: level_id for user_id, level_id in users_level}
                    cache.set('users_level', users_level, 1800)
            except Exception as e:
                print(e)



import os
from dotenv import load_dotenv
load_dotenv()
from telegram import Bot
from telegram.ext import Application
from bot.core.base import Base
import inspect
from bot.utils import root_path,FileFinder
from telegram.ext import PicklePersistence
import importlib
from pathlib import Path
import logging

class TelegramBot:
    def __init__(
            self,
            token: str = None,
            admins: list[int] = None
    ):
            token = token or os.environ.get('BOT_TOKEN')
            if admins is None:
                admins: list = os.getenv('ADMIN_IDS').split(' ')
                admins = list(map(lambda x: int(x.strip()), admins))
            persistence = PicklePersistence(filepath="bot_persistence",update_interval=5)
            proxy = os.getenv('PROXY')
            proxy_enable = bool(int(os.getenv('PROXY_ENABLE')))
            if proxy_enable:
                self.app = Application.builder().token(token).proxy(proxy).persistence(persistence).build()
            else:
                self.app = Application.builder().token(token).persistence(persistence).build()
            self.bot:Bot = self.app.bot
            self.admins = admins
            self._node_dir=os.getenv('PANEL_DIR')+'/roles'
            self.app.add_error_handler(self.error_handler)
    async def error_handler(self,update, context):
        # ثبت اطلاعات خطا در لاگ
        logging.error("Exception occurred", exc_info=context.error)

    def autodiscover(self):
        path=root_path()
        ff=FileFinder(f"{path}/{self._node_dir}")
        # Get all Python files in the nodes directory
        python_files = ff.find_by_extension('.py')
        for file_path in python_files:
            # Convert file path to module path
            relative_path = Path(file_path).relative_to(path)
            module_path = str(relative_path).replace('/', '.').replace('.py', '')
            # Load the module
            module = self.__load_module_from_str(module_path)

            if module:
                # Find all Base subclasses in the module
                subclasses_in_module = [
                    cls for name, cls in inspect.getmembers(module, inspect.isclass)
                    if issubclass(cls, Base) and cls is not Base
                ]
                # Initialize and process each subclass
                for cls in subclasses_in_module:
                    print(f"Loading panel {cls.__name__}")
                    instance:Base = cls()
                    instance.build(self.bot,self.admins)
                    for handler in instance._handlers:
                        self.app.add_handler(handler)

    async def stop(self):
        """Stops the setting and cleans up resources"""
        if self.app:
            await self.app.stop()
            await self.app.shutdown()

    def run(self):
        self.autodiscover()
        self.app.run_polling()


    def __load_module_from_str(self,module_path: str):
        """
        Load a module from a string path.
        Example: 'nodes.admin.commands' will load the commands module
        """
        try:

            return importlib.import_module(module_path)
        except ImportError as e:
            logging.error(e)
            return None


from django.core.management import call_command
from django.core.management.base import BaseCommand
from pathlib import Path
from django.conf import settings
import os,json
from bot.models import User
from dotenv import load_dotenv
from bot.core.used_texts_key import keys
load_dotenv()

role_class='''
from bot.core.base import Base, MH
class Node{}(Base):
    def __init__(self):
        super().__init__('{}')
        self.menu_btns = []
        self.menu_btns_pattern = 2
        # self.add_middleware(middleware_func)
        # self.add_command('start','start')
        # self.add_command('help','help')

    def n_start(self):
        e = MH()
        e.filters = '/start'
        async def act():
            await self.start_action()

        e.callback = act
        self.node(e)
'''

class Command(BaseCommand):
    help = 'Creates bot_panel directory structure in Django root'

    def add_arguments(self, parser):
        parser.add_argument(
            '--force',
            action='store_true',
            help='Force creation even if directory exists',
        )

    def handle(self, *args, **options):
        # Get Django project root (where manage.py is located)
        django_root = Path(settings.BASE_DIR)
        call_command('bot_env', add_missing=True, show_values=True)
        load_dotenv()
        bot_panel_dir = django_root /os.getenv('PANEL_DIR')

        try:
            # Create main bot_panel directory
            bot_panel_dir.mkdir(exist_ok=True)
            self.stdout.write(
                self.style.SUCCESS(f'Created directory: {bot_panel_dir}')
            )

            # Create __init__.py
            init_file = bot_panel_dir / '__init__.py'
            init_file.touch()
            self.stdout.write(
                self.style.SUCCESS(f'Created file: {init_file}')
            )

            # Create core subdirectory
            core_dir = bot_panel_dir / 'texts'
            core_dir.mkdir(exist_ok=True)
            self.stdout.write(
                self.style.SUCCESS(f'Created directory: {core_dir}')
            )
            core_dir = bot_panel_dir / 'roles'
            core_dir.mkdir(exist_ok=True)
            self.stdout.write(
                self.style.SUCCESS(f'Created directory: {core_dir}')
            )
            for i in User.LEVELS:
                target_dir = bot_panel_dir/ 'roles' / i
                target_dir.mkdir(exist_ok=True)

                file_path = target_dir / '__init__.py'
                if not file_path.exists():
                    txt=role_class.format(i.title(),i)
                    with open(file_path, 'w') as f:
                        f.write(txt)
                    self.stdout.write(
                        self.style.SUCCESS(f'Created file: {file_path}')
                    )

                self.stdout.write(
                    self.style.SUCCESS(f'Created directory: {core_dir}')
                )

            for i in User.LANGUAGES:
                target_dir = bot_panel_dir/ 'texts' / i
                target_dir.mkdir(exist_ok=True)
                file_path = target_dir / 'text.json'
                if not file_path.exists():
                    with open(file_path,'w') as f:
                        f.write(json.dumps({i: '' for i in keys}))
                    self.stdout.write(
                        self.style.SUCCESS(f'Created file: {file_path}')
                    )
                self.stdout.write(
                    self.style.SUCCESS(f'Created directory: {core_dir}')
                )
            #     file_path = core_dir / filename
            #     if not file_path.exists() or force:
            #         file_path.touch()
            #         self.stdout.write(
            #             self.style.SUCCESS(f'Created file: {file_path}')
            #         )

            self.stdout.write(
                self.style.SUCCESS(
                    '\nbot_panel directory structure created successfully!'
                )
            )

        except Exception as e:
            self.stdout.write(
                self.style.ERROR(f'Error creating bot_panel directory: {str(e)}')
            )


import os
from time import sleep
from telegram.ext.filters import *
from django.core.cache import cache
from threading import Thread

class LevelFilter(BaseFilter):
    __slots__ = ("parent","admins")

    def __init__(self, parent,admins):
        super().__init__()
        self.parent = parent
        self.admins=admins

    def check_update(self, update):
        out= self.filter(update)
        return out

    def filter(self, update) -> bool:
        if not update.message:
            return False
        try:

            level = self.parent._level
            admins = self.admins
            user_id = update.message.from_user.id
            if user_id in admins:
                if level=='admin':
                    print(f"is admin")
                    return True
                return False

            cache_key=f"users_level"
            users_level = cache.get(cache_key)
            if users_level:
                if users_level.get(user_id):
                    return users_level.get(user_id)==level
            else:
                Thread(target=self.parent._set_users_level,kwargs={'empty_cache':True}).start()
                sleep(1)
                users_level = cache.get(cache_key)
                print(f"{user_id},{ users_level.get(user_id)},{users_level=}",level)
                if users_level:
                    return users_level.get(user_id,os.getenv('DEFAULT_LEVEL')) == level
                else:return False

            return level ==os.getenv('DEFAULT_LEVEL')
        except Exception as e:
            print(e)
            return False

class MatchText(BaseFilter):
    __slots__ = ('text',"parent",'exact')

    def __init__(self,text, parent,exact=True):
        super().__init__()
        self.parent = parent
        self.exact=exact
        self.text=text


    def check_update(self, update):
        out= self.filter(update)
        return out

    def filter(self, update) -> bool:
        if not update.effective_message:
            return False
        try:
            text = update.effective_message.text
            ttext=self.text
            if self.exact:
                return ttext==text
            else:
                return ttext in text
        except Exception as e:
            print(e)
        return False


class IsConversation(BaseFilter):
    __slots__ = ("parent","status","default")

    def __init__(self,parent,status,default=True):
        super().__init__()
        self.parent = parent
        self.status = status
        self.default=default


    def check_update(self, update):
        out= self.filter(update)
        return out

    def filter(self, update) -> bool:
        if not update.effective_message:
            return False
        try:
            return self.parent.context.user_data.get('is_conversation',self.default)==self.status
        except Exception as e:
            print(e)
        return self.default==self.status


class StatFilter(BaseFilter):
    __slots__ = ("parent","stat")

    def __init__(self,parent,stat):
        super().__init__()
        self.parent = parent
        self.stat = stat


    def check_update(self, update):
        out= self.filter(update)
        return out

    def filter(self, update) -> bool:
        if not update.effective_message:
            return False
        try:
            return self.parent.context.user_data.get('stat',{}).get('filter')==self.stat
        except Exception as e:
            print(e)
        return False




import os

from celery import shared_task
import telegram
from time import sleep
from asgiref.sync import  sync_to_async
from bot.models import User
import asyncio
from functools import wraps
from django.conf import settings

def run_async(func):
    @wraps(func)
    def wrapper(*args, **kwargs):
        loop = asyncio.get_event_loop()
        return loop.run_until_complete(func(*args, **kwargs))
    return wrapper
@shared_task(queue='broadcast')
@run_async
async def broadcast_message_to_users(users_id,message_id,from_user_id,**kwargs):
    """
    Celery task to send message to all active users
    """
    bot = telegram.Bot(settings.TELEGRAM_BOT_TOKEN)
    success_count = 0
    failed_count = 0
    for user_id in users_id:
        try:
            await bot.copy_message(
                chat_id=user_id,
                from_chat_id=from_user_id,
                message_id=message_id,**kwargs
            )
            success_count += 1
        except telegram.error.Forbidden as e:
            print(e)
            await sync_to_async(User.objects.filter(user_id=user_id).update)(is_active=False)
            failed_count += 1
            continue
        except telegram.error.RetryAfter as e:
            print(e)

            retry_after=e.retry_after
            sleep(retry_after+5)
            await bot.copy_message(
                chat_id=user_id,
                from_chat_id=from_user_id,
                message_id=message_id, **kwargs
            )
        except Exception as e:
            print(e)
            failed_count += 1
            continue
        sleep(3)

    msg=f'''
        total_users: {len(users_id)}
        success_count: {success_count}
        failed_count: {failed_count}
    '''
    await bot.send_message(
        chat_id=int(os.getenv('ADMIN_IDS').split(' ')[0]),
        text=msg,
    )



from pathlib import Path
from bot.component.button import ButtonBuilder

def button(data: list | dict, pattern: int | tuple, reverse: bool = False, close: bool = False, level=1):
    new_data = data.copy()
    if isinstance(data, dict):
        for key, value in data.items():
            if isinstance(value, dict):
                if 'R' not in value:
                    value['R'] = level
                new_data[key] = value
    return ButtonBuilder(new_data, pattern, reverse, close).build()


def get_static_file_bytes(static_path):
    return Path(static_path).read_bytes()


def get_media_type(file_extension):
    """Determine media type based on file extension"""
    photo_extensions = {'jpg', 'jpeg', 'png', 'webp'}
    video_extensions = {'mp4', 'avi', 'mov', 'mkv'}
    audio_extensions = {'mp3', 'wav', 'ogg', 'm4a'}
    document_extensions = {'pdf', 'doc', 'docx', 'txt', 'zip', 'rar'}

    ext = file_extension.lower()

    if ext in photo_extensions:
        return 'photo'
    elif ext in video_extensions:
        return 'video'
    elif ext in audio_extensions:
        return 'audio'
    elif ext in document_extensions:
        return 'document'
    else:
        return 'document'


def status_maker(status):
    if status == 'completed':
        return f'✅{status}'
    elif status == 'confirmed':
        return f'✅{status}'
    elif status == 'canceled':
        return f'❌{status}'
    elif status == 'pending':
        return f'⏳{status}'
    elif status == 'processing':
        return f'🔄{status}'
    elif status == 'refund':
        return f'💸{status}'
    elif status == 'failed':
        return f'❌{status}'
    elif status == 'rejected':
        return f'❌{status}'

import os

from pathlib import Path
import os
from pathlib import Path
from django.conf import settings

def root_path():
    return settings.BASE_DIR

class FileFinder:
    def __init__(self, directory):
        self.directory = directory

    def find_all_files(self):
        """Returns a list of all files in the directory and subdirectories"""
        files = []
        for root, dirs, filenames in os.walk(self.directory):
            for filename in filenames:
                files.append(os.path.join(root, filename))
        return files

    def find_by_extension(self, extension):
        """Returns a list of files with specific extension"""
        files = []
        for file in self.find_all_files():
            if file.endswith(extension):
                files.append(file)
        return files

    def get_file_count(self):
        """Returns total number of files"""
        return len(self.find_all_files())


